from gettext import textdomain, dngettext, ngettext, bindtextdomain, dgettext, gettext
try:
    from gettext import bind_textdomain_codeset
except ImportError:
    bind_textdomain_codeset = None
import os, sys, re

__version__ = "0.108"

__all__ = [
    'l_nolocales',
    'l_dry',
    'l_dir',
    'l_lang',
    'ltd',
    'l',
    'ln',
    'lp',
    'lnp',
    'ld',
    'ldn',
    'ldp',
    'ldnp',
    'N_',
    'Nn_',
    'Np_',
    'Nnp_',
    'Nd_',
    'Ndn_',
    'Ndp_',
    'Ndnp_',
]

global dry, nowrite, nolocales, dir 
dry, nowrite, nolocales, dir = None, None, None, None
tds = {}

def l_nolocales(x):
    global nolocales
    nolocales = x
def l_dry(d, n=False):
    global dry, nowrite, nolocales
    dry, nowrite, nolocales = d, n, bool(d)
def l_dir(d): 
    global dir
    if not os.path.isdir(d): raise OSError("%s: No such file or directory" % d)
    dir = d

def _die(msg):
    sys.stderr.write(msg + '\n')
    exit(1)

def gettext_escape(content):
    content = content.replace('\n', '\\n')
    content = content.replace('"', '\"')
    return content

def l_lang(primary):
    for var in 'LANG', 'LANGUAGE', 'LC_ALL':
        os.environ.update({var: primary})

def wd(data):
    if nowrite: return
    f = open(dry, 'a')
    f.write('\n'.join(data) + "\n\n")
    f.close()

def sprintf(string, *args):
    args = list(args)
    for m in re.finditer(r'%(\d+)\$([scboxXuidfegEG])', string):
        substr = m.string[m.start():m.end()]
        groups = m.groups()
        string = string.replace(substr, ('%'+groups[1]) % args[int(groups[0])-1])
    count = len(re.findall(r'%%|%(?:\d+\$)?(?:[-+\'#0 ]*)(?:\*\d+\$|\*|\d+)?(\.(?:\*\d+\$|\*|\d+))?(?:[scboxXuidfegEG])', string))
    args = tuple(args[:count])
    return string % args

def l(id, *args):
    return ldnp('', None, id, None, None, *args)
def ln(id, idp, n, *args):
    return ldnp('', None, id, idp, n, *args)
def lp(ctxt, id, *args):
    return ldnp('', ctxt, id, None, None, *args)
def lnp(ctxt, id, idp, n, *args):
    return ldnp('', ctxt, id, idp, n, *args)
def ld(td, id, *args):
    return ldnp(td, None, id, None, None, *args)
def ldn(td, id, idp, n, *args):
    return ldnp(td, None, id, idp, n, *args)
def ldp(td, ctxt, id, *args):
    return ldnp(td, ctxt, id, None, None, *args)

def ldnp(td, ctxt, id, idp, n, *args):
    if not len(id): return id
    if not dir and not nolocales:
        _die("please set a locale directory with l_dir() before using other translate functions")

    if idp is not None: args = (n,) + args
    is_plural = idp is not None
    out = None
    if dry:
        if not nowrite:
            save = []
            if td: save.append(' # domain: '+td)
            if ctxt: save.append('msgctxt: "'+gettext_escape(ctxt)+'"')
            save.append('msgid "'+gettext_escape(id)+'"')
            if idp: save.append('msgid_plural "'+gettext_escape(idp)+'"')
            wd(save)
        out = (idp if is_plural and n != 1 else id) % args
    else:
        key = (ctxt + '\x04' + id) if ctxt else id
        keyp = (ctxt + '\x04' + idp) if ctxt and idp else idp
        if   td and is_plural:     raw = dngettext(td, key, keyp, n)
        elif td and not is_plural: raw = dgettext(td, key)
        elif is_plural:            raw = ngettext(key, keyp, n)
        else:                      raw = gettext(key)

        # gettext returns the input string untouched when no translation
        # exists, so for a pgettext-style lookup we have to strip the
        # "ctxt\x04" prefix ourselves to fall back to the plain msgid.
        if ctxt and raw.startswith(ctxt + '\x04'):
            raw = raw[len(ctxt) + 1:]

        out = sprintf(raw, *args)
    return out

# Deferred-translation markers — runtime no-ops so strings can be declared
# before a user locale is known (class-construction time, DSLs, constants)
# and still show up in the extracted .pot. Pass the stored msgid(s) through
# the real l*() at render time. See Locale::Simple POD for the full story.
def N_(msgid):
    return msgid
def Nn_(sg, pl):
    return (sg, pl)
def Np_(ctxt, msgid):
    return msgid
def Nnp_(ctxt, sg, pl):
    return (sg, pl)
def Nd_(td, msgid):
    return msgid
def Ndn_(td, sg, pl):
    return (sg, pl)
def Ndp_(td, ctxt, msgid):
    return msgid
def Ndnp_(td, ctxt, sg, pl):
    return (sg, pl)


def ltd(td):
    if td not in tds:
        bindtextdomain(td, dir)
        if bind_textdomain_codeset:
            bind_textdomain_codeset(td, 'utf-8')
        tds[td] = 1
    textdomain(td)



l("PY N plain")
ln("PY N one", "PY N many", 4)
lp("PY_ctx", "PY N ctx plain")
lnp("PY_ctx", "PY N ctx one", "PY N ctx many", 4)
ld("PY_dom", "PY N dom plain")
ldn("PY_dom", "PY N dom one", "PY N dom many", 4)
ldp("PY_dom", "PY_ctx", "PY N dom ctx plain")
ldnp("PY_dom", "PY_ctx", "PY N dom ctx one", "PY N dom ctx many", 4)



import sys, os

global err
err = 0

def test(func): 
    global err
    print(func.__name__ + '...')
    try: 
        func()
        print('\033[32mOK\033[0m\n')
    except BaseException as e:
        err = 1
        print('\033[31m' + str(e) + '\033[0m')


def testeq(one, two):
    sys.stdout.write("Testing equality (%s == %s)..." % (repr(one), repr(two)))
    if one != two: raise ValueError("The two values do not match")
    else: print('OK')

@test
def test_import():
    from locale_simple import __all__
    testeq(type(__all__), list)

from locale_simple import *

@test
def setup():
    l_dir(os.path.dirname(os.path.abspath(__file__)) + '/../t/data/locale')
    ltd('test')
    l_lang('de_DE')

@test
def translate():
    testeq(
            l('Hello'),
            'Hallo'
            )

    testeq(
            ln("You have %d message","You have %d messages",4),
            'Du hast 4 Nachrichten'
            )

    testeq(
            ln("You have %d message","You have %d messages",1),
            'Du hast 1 Nachricht'
            )

    testeq(
            ln("You have %d message of %s","You have %d messages of %s",1,'harry'),
            'Du hast 1 Nachricht von harry'
            )

    testeq(
            ln("You have %d message of %s","You have %d messages of %s",4,'harry'),
            'Du hast 4 Nachrichten von harry'
            )

    testeq(
            ln('%2$s brought %1$d message','%2$s brought %1$d messages',1,'harry'),
            '1 Nachricht gebracht von harry'
            )

    testeq(
            ln('%2$s has %1$d message','%2$s has %1$d messages',4,'harry'),
            'harry hat 4 Nachrichten'
            )

    testeq(
            l("Change order test %s %s", 'one', 'two'),
            "Andere Reihenfolge hier two one"
            )

    testeq(
            l('Other change order test %s %s %s', 1, 2, 3),
            'Verhalten aus http://perldoc.perl.org/functions/sprintf.html 3 1 1'
            )

    testeq(
            lp("alien","Hello"),
            "Hallo Ausserirdischer"
            )

exit(err)



N_("PY N plain")
Nn_("PY N one", "PY N many")
Np_("PY_ctx", "PY N ctx plain")
Nnp_("PY_ctx", "PY N ctx one", "PY N ctx many")
Nd_("PY_dom", "PY N dom plain")
Ndn_("PY_dom", "PY N dom one", "PY N dom many")
Ndp_("PY_dom", "PY_ctx", "PY N dom ctx plain")
Ndnp_("PY_dom", "PY_ctx", "PY N dom ctx one", "PY N dom ctx many")



import sys, os, tempfile

global err
err = 0

def test(func):
    global err
    print(func.__name__ + '...')
    try:
        func()
        print('\033[32mOK\033[0m\n')
    except BaseException as e:
        err = 1
        print('\033[31m' + str(e) + '\033[0m')


def testeq(one, two):
    sys.stdout.write("Testing equality (%s == %s)..." % (repr(one), repr(two)))
    if one != two: raise ValueError("The two values do not match")
    else: print('OK')


@test
def test_import():
    from locale_simple import __all__
    testeq(type(__all__), list)
    # API parity with Perl/JS siblings
    for name in ('l', 'ln', 'lp', 'lnp', 'ld', 'ldn', 'ldp', 'ldnp',
                 'N_', 'Nn_', 'Np_', 'Nnp_', 'Nd_', 'Ndn_', 'Ndp_', 'Ndnp_',
                 'l_dir', 'l_lang', 'l_dry', 'l_nolocales', 'ltd'):
        if name not in __all__:
            raise ValueError("%s missing from __all__" % name)


from locale_simple import *

LOCALE_DIR = os.path.dirname(os.path.abspath(__file__)) + '/../t/data/locale'


@test
def setup():
    l_dir(LOCALE_DIR)
    ltd('test')
    l_lang('de_DE')


@test
def translate():
    testeq(l('Hello'), 'Hallo')

    testeq(
        ln("You have %d message", "You have %d messages", 4),
        'Du hast 4 Nachrichten'
    )
    testeq(
        ln("You have %d message", "You have %d messages", 1),
        'Du hast 1 Nachricht'
    )
    testeq(
        ln("You have %d message", "You have %d messages", 0),
        'Du hast 0 Nachrichten'
    )
    testeq(
        ln("You have %d message of %s", "You have %d messages of %s", 1, 'harry'),
        'Du hast 1 Nachricht von harry'
    )
    testeq(
        ln("You have %d message of %s", "You have %d messages of %s", 4, 'harry'),
        'Du hast 4 Nachrichten von harry'
    )
    testeq(
        ln('%2$s brought %1$d message', '%2$s brought %1$d messages', 1, 'harry'),
        '1 Nachricht gebracht von harry'
    )
    testeq(
        ln('%2$s has %1$d message', '%2$s has %1$d messages', 4, 'harry'),
        'harry hat 4 Nachrichten'
    )
    testeq(
        l("Change order test %s %s", 'one', 'two'),
        "Andere Reihenfolge hier two one"
    )
    testeq(
        l('Other change order test %s %s %s', 1, 2, 3),
        'Verhalten aus http://perldoc.perl.org/functions/sprintf.html 3 1 1'
    )
    testeq(lp("alien", "Hello"), "Hallo Ausserirdischer")
    testeq(l(''), '')


@test
def context_plural():
    # lnp: context + plural. No context-plural msgid in the fixture, so we
    # just verify it doesn't blow up and falls back to the msgid.
    testeq(
        lnp('some_ctxt', 'You have %d message', 'You have %d messages', 1),
        'You have 1 message'
    )
    testeq(
        lnp('some_ctxt', 'You have %d message', 'You have %d messages', 4),
        'You have 4 messages'
    )


@test
def domain_switching():
    # ld / ldn / ldp / ldnp — the "other" domain has different translations
    # for the same msgids.
    ltd('othertest')  # bind the domain
    ltd('test')       # switch back to default, so ld() proves it's not
                      # reading from the current textdomain

    testeq(ld('othertest', 'Hello'), 'Anderes Hallo')
    testeq(
        ldn('othertest', 'You have %d message', 'You have %d messages', 4),
        'Du hast 4 AndereNachrichten'
    )
    testeq(
        ldn('othertest', 'You have %d message', 'You have %d messages', 1),
        'Du hast 1 AndereNachricht'
    )
    # Non-existent domain: falls through to msgid
    testeq(ld('nonexistent', 'Hello'), 'Hello')
    # ldp with unknown context: fall back to msgid
    testeq(ldp('othertest', 'no_such_ctxt', 'Hello'), 'Hello')


@test
def dry_run():
    with tempfile.NamedTemporaryFile(mode='w', delete=False, suffix='.po') as tf:
        dry_file = tf.name
    try:
        l_dry(dry_file)
        # l_dry flips nolocales=True, so l() should pass through formatting
        # without hitting gettext.
        testeq(l('Hello %s', 'World'), 'Hello World')
        testeq(
            ln('One %d', 'Many %d', 1),
            'One 1'
        )
        testeq(
            ln('One %d', 'Many %d', 5),
            'Many 5'
        )
        with open(dry_file) as f:
            content = f.read()
        # Confirm msgids were appended
        if 'msgid "Hello %s"' not in content:
            raise ValueError("dry-run file did not capture Hello msgid")
        if 'msgid "One %d"' not in content:
            raise ValueError("dry-run file did not capture plural msgid")
        if 'msgid_plural "Many %d"' not in content:
            raise ValueError("dry-run file did not capture plural_id")
    finally:
        os.unlink(dry_file)
        # Reset state for any later test
        l_dry(None)
        l_dir(LOCALE_DIR)
        ltd('test')
        l_lang('de_DE')


@test
def nolocales_mode():
    # With l_nolocales(True) + no dir set, l() should still work as identity.
    # We simulate it by flipping the flag — formatting-only mode.
    from locale_simple import l_nolocales
    # Already in a known-good state thanks to dry_run teardown; no need to
    # unset l_dir. Just flip the flag and confirm no exception.
    l_nolocales(True)
    # Should still produce a translated result since dir is set and gettext
    # returns the actual translation.
    testeq(l('Hello'), 'Hallo')
    l_nolocales(False)


@test
def language_switching_lifecycle():
    # Full round-trip: load, translate, switch lang, translate again.
    l_dir(LOCALE_DIR)
    ltd('test')
    l_lang('de_DE')
    testeq(l('Hello'), 'Hallo')

    # Switching to a language with no .mo falls back to msgid.
    l_lang('xx_YY')
    # gettext caches — it may still return 'Hallo' from this process. The
    # lifecycle we care about is "no crash when switching".
    result = l('Hello')
    if result not in ('Hello', 'Hallo'):
        raise ValueError("Unexpected result after l_lang switch: %r" % result)

    # Switch back.
    l_lang('de_DE')


@test
def deferred_markers():
    # Identity contract — the whole point of the N*_ family is that
    # the caller gets back exactly what it put in, so the scraper can
    # see the msgid at build time while runtime defers the lookup.
    testeq(N_("Hello"), "Hello")
    testeq(Np_("menu", "Open"), "Open")
    testeq(Nd_("app", "Hi"), "Hi")
    testeq(Ndp_("app", "menu", "Open"), "Open")

    # Plural variants return (sg, pl) so caller can unpack into ln(...).
    testeq(Nn_("1 file", "%d files"), ("1 file", "%d files"))
    testeq(Nnp_("ctx", "1 file", "%d files"), ("1 file", "%d files"))
    testeq(Ndn_("dom", "1 file", "%d files"), ("1 file", "%d files"))
    testeq(Ndnp_("dom", "ctx", "1 file", "%d files"), ("1 file", "%d files"))

    # Round-trip: store via N_, render via l() — ensures what N_ hands
    # back is exactly what the runtime l() expects later.
    label = N_("Hello")
    testeq(l(label), "Hallo")

    sg, pl = Nn_("You have %d message", "You have %d messages")
    testeq(ln(sg, pl, 1), "Du hast 1 Nachricht")
    testeq(ln(sg, pl, 4), "Du hast 4 Nachrichten")


exit(err)
